/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: clkwiz.h
 * 
 * Date: 02.12.2013
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    Implements functions for handling Video Timing Controller
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/

#ifndef _CLKWIZ_HEADER_FILE_
#define _CLKWIZ_HEADER_FILE_

#include "xil_types.h"
#include "xil_io.h"


#define CLKWIZ_SW_RESET_OFFSET          0x000
#define CLKWIZ_STATUS_OFFSET            0x004
#define CLKWIZ_VCO_OFFSET               0x200
#define CLKWIZ_CLKFBOUT_PHASE_OFFSET    0x204

#define CLKWIZ_CLKOUT0_DIVIDE_OFFSET    0x208
#define CLKWIZ_CLKOUT0_PHASE_OFFSET     0x20C
#define CLKWIZ_CLKOUT0_DUTY_OFFSET      0x210

#define CLKWIZ_CLKOUT1_DIVIDE_OFFSET    0x214
#define CLKWIZ_CLKOUT1_PHASE_OFFSET     0x218
#define CLKWIZ_CLKOUT1_DUTY_OFFSET      0x21C

#define CLKWIZ_CLKOUT2_DIVIDE_OFFSET    0x220
#define CLKWIZ_CLKOUT2_PHASE_OFFSET     0x224
#define CLKWIZ_CLKOUT2_DUTY_OFFSET      0x228
                                        
#define CLKWIZ_CLKOUT3_DIVIDE_OFFSET    0x22C
#define CLKWIZ_CLKOUT3_PHASE_OFFSET     0x230
#define CLKWIZ_CLKOUT3_DUTY_OFFSET      0x234
                                        
#define CLKWIZ_CLKOUT4_DIVIDE_OFFSET    0x238
#define CLKWIZ_CLKOUT4_PHASE_OFFSET     0x23C
#define CLKWIZ_CLKOUT4_DUTY_OFFSET      0x240
                                        
#define CLKWIZ_CLKOUT5_DIVIDE_OFFSET    0x244
#define CLKWIZ_CLKOUT5_PHASE_OFFSET     0x248
#define CLKWIZ_CLKOUT5_DUTY_OFFSET      0x24C
                                        
#define CLKWIZ_CLKOUT6_DIVIDE_OFFSET    0x250
#define CLKWIZ_CLKOUT6_PHASE_OFFSET     0x254
#define CLKWIZ_CLKOUT6_DUTY_OFFSET      0x258

#define CLKWIZ_LOAD_OFFSET              0x25C


#define CLKWIZ_LOCKED 0x01

#define CLKOUT0   0
#define CLKOUT1   1
#define CLKOUT2   2
#define CLKOUT3   3
#define CLKOUT4   4
#define CLKOUT5   5
#define CLKOUT6   6

#define CLKWIZ_WriteReg32(BaseAddress, RegOffset, Data) \
    Xil_Out32((BaseAddress) + (RegOffset), (u32)(Data))

#define CLKWIZ_ReadReg32(BaseAddress, RegOffset) \
    Xil_In32((BaseAddress) + (RegOffset))


// zynq 20 speed grade                     |   3     |   -2    | 1C/-1I/-1LI |  -1Q
// -------------------------------------------------------------------------------------
// MMCM_FVCOMIN Minimum MMCM VCO frequency | 600.00  | 600.00  | 600.00      | 600.00    MHz
// MMCM_FVCOMAX Maximum MMCM VCO frequency | 1600.00 | 1440.00 | 1200.00     | 1200.00   MHz
// PLL_FVCOMIN Minimum PLL VCO frequency   | 800.00  | 800.00  | 800.00      | 800.00    MHz
// PLL_FVCOMAX Maximum PLL VCO frequency   | 2133.00 | 1866.00 | 1600.00     | 1600.00   MHz
    

// clk_vco = (clk_in * clk_mul) / clk_div
// clk_out = clk_vco / clk_out_dif
    

// MMCM CLKOUT0
struct struct_mmcm_clkout0_t
{
    float clk_out;
    u8    clk_div;      // 1    - 106
    float clk_mul;      // 2.0  - 64.0
    float clk_out_div;  // 2.0  - 128.0
};
typedef struct struct_mmcm_clkout0_t mmcm_clkout0_t;

// MMCM CLKOUT1 - CLKOUT6
struct struct_mmcm_clkout1_6_t
{
    float clk_out;
    u8    clk_div;      // 1    - 106
    float clk_mul;      // 2.0  - 64.0
    u8    clk_out_div;  // 2    - 128
};
typedef struct struct_mmcm_clkout1_6_t mmcm_clkout1_6_t;

// CLK_WIZARD
struct struct_clkwiz_t
{
    u32               base_addr;
    float             clk_in;
    float             clk_vco;
    float             clk_vco_min;
    float             clk_vco_max;
    mmcm_clkout0_t    clkout0;
    mmcm_clkout1_6_t  clkout1;
    mmcm_clkout1_6_t  clkout2;
    mmcm_clkout1_6_t  clkout3;
    mmcm_clkout1_6_t  clkout4;
    mmcm_clkout1_6_t  clkout5;
    mmcm_clkout1_6_t  clkout6;
};
typedef struct struct_clkwiz_t clkwiz_t;


#define CLKWIZ_TAB_LEN    9

#define CLKWIZ_148_5_MHZ  0
#define CLKWIZ_128_44_MHZ 1
#define CLKWIZ_135_MHZ    2
#define CLKWIZ_119_23_MHZ 3
#define CLKWIZ_108_MHZ    4
#define CLKWIZ_74_25_MHZ  5
#define CLKWIZ_65_MHZ     6 
#define CLKWIZ_40_5_MHZ   7 
#define CLKWIZ_25_19_MHZ  8



inline u32 get_clkwiz_stat(u32 ctrl_base);
inline u32 is_clkwiz_locked(u32 ctrl_base);
void inline clkwiz_reset (u32 ctrl_base);


void clkwiz_set_vco(clkwiz_t *clkwiz);
void clkwiz_set_clkout0(clkwiz_t *clkwiz);

void clkwiz_clkout0_lookup(mmcm_clkout0_t *clkout0, u8 sel);


#define clkwiz_update(base_addr) \
    CLKWIZ_WriteReg32(base_addr, CLKWIZ_LOAD_OFFSET, 0x00000007); \
    CLKWIZ_WriteReg32(base_addr, CLKWIZ_LOAD_OFFSET, 0x00000002)



#endif /* _CLKWIZ_HEADER_FILE_*/
